@@ -1,9 +1,9 @@
-/* $Id: browser.c,v 1.9 2005/12/08 02:47:10 dolorous Exp $ */
+/* $Id: browser.c,v 1.25 2006/03/23 23:17:27 dolorous Exp $ */
 /**************************************************************************
  *   browser.c                                                            *
  *                                                                        *
  *   Copyright (C) 2001-2004 Chris Allegretta                             *
- *   Copyright (C) 2005 David Lawrence Ramsey                             *
+ *   Copyright (C) 2005-2006 David Lawrence Ramsey                        *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
  *   the Free Software Foundation; either version 2, or (at your option)  *
@@ -30,25 +30,35 @@
 
 #ifndef DISABLE_BROWSER
 
+static char **filelist = NULL;
+	/* The list of files to display in the browser. */
+static size_t filelist_len = 0;
+	/* The number of files in the list. */
+static int width = 0;
+	/* The number of columns to display the list in. */
+static int longest = 0;
+	/* The number of columns in the longest filename in the list. */
+static size_t selected = 0;
+	/* The currently selected filename in the list. */
+
 /* Our browser function.  path is the path to start browsing from.
  * Assume path has already been tilde-expanded. */
 char *do_browser(char *path, DIR *dir)
 {
-    int kbinput, longest, selected, width;
-    bool meta_key, func_key, old_const_update = ISSET(CONST_UPDATE);
-    size_t numents;
-    char **filelist, *retval = NULL;
+    int kbinput;
+    bool meta_key, func_key;
+    bool old_const_update = ISSET(CONST_UPDATE);
+    char *ans = mallocstrcpy(NULL, "");
+	/* The last answer the user typed on the statusbar. */
+    char *retval = NULL;
 
     curs_set(0);
     blank_statusbar();
-    bottombars(browser_list);
-    wnoutrefresh(bottomwin);
-
 #if !defined(DISABLE_HELP) || !defined(DISABLE_MOUSE)
-    /* Set currshortcut so the user can click in the shortcut area, and
-     * so the browser help screen will come up. */
     currshortcut = browser_list;
 #endif
+    bottombars(browser_list);
+    wnoutrefresh(bottomwin);
 
     UNSET(CONST_UPDATE);
 
@@ -56,8 +66,10 @@ char *do_browser(char *path, DIR *dir)
 	/* We go here after the user selects a new directory. */
 
     kbinput = ERR;
-    selected = 0;
+    meta_key = FALSE;
+    func_key = FALSE;
     width = 0;
+    selected = 0;
 
     path = mallocstrassn(path, get_full_path(path));
 
@@ -65,22 +77,20 @@ char *do_browser(char *path, DIR *dir)
     assert(path != NULL && path[strlen(path) - 1] == '/');
 
     /* Get the list of files. */
-    filelist = browser_init(path, &longest, &numents, dir);
+    browser_init(path, dir);
 
     assert(filelist != NULL);
 
     /* Sort the list. */
-    qsort(filelist, numents, sizeof(char *), diralphasort);
+    qsort(filelist, filelist_len, sizeof(char *), diralphasort);
 
     titlebar(path);
 
     do {
 	bool abort = FALSE;
-	int j, col = 0, editline = 0, fileline;
-	int filecols = 0;
-	    /* Used only if width == 0, to calculate the number of files
-	     * per row below. */
 	struct stat st;
+	int i;
+	size_t fileline;
 	char *new_path;
 	    /* Used by the "Go To Directory" prompt. */
 #ifndef DISABLE_MOUSE
@@ -90,7 +100,7 @@ char *do_browser(char *path, DIR *dir)
 	check_statusblank();
 
 	/* Compute the line number we're on now, so that we don't divide
-	 * by zero later. */
+	 * by 0. */
 	fileline = selected;
 	if (width != 0)
 	    fileline /= width;
@@ -104,16 +114,16 @@ char *do_browser(char *path, DIR *dir)
 		/* If we clicked in the edit window, we probably clicked
 		 * on a file. */
 		if (wenclose(edit, mevent.y, mevent.x)) {
-		    int old_selected = selected;
+		    size_t old_selected = selected;
 
 		    /* Subtract out the size of topwin. */
 		    mevent.y -= 2 - no_more_space();
 
 		    /* longest is the width of each column.  There are
 		     * two spaces between each column. */
-		    selected = (fileline / editwinrows) * editwinrows *
-			width + mevent.y * width + mevent.x /
-			(longest + 2);
+		    selected = (fileline / editwinrows) * (editwinrows *
+			width) + (mevent.y * width) + (mevent.x /
+			(longest + 2));
 
 		    /* If they clicked beyond the end of a row, select
 		     * the end of that row. */
@@ -123,10 +133,11 @@ char *do_browser(char *path, DIR *dir)
 		    /* If we're off the screen, reset to the last item.
 		     * If we clicked the same place as last time, select
 		     * this name! */
-		    if (selected > numents - 1)
-			selected = numents - 1;
+		    if (selected > filelist_len - 1)
+			selected = filelist_len - 1;
 		    else if (old_selected == selected)
-			/* Put back the 'select' key. */
+			/* Put back the "Select" key, so that the file
+			 * is selected. */
 			unget_kbinput(NANO_ENTER_KEY, FALSE, FALSE);
 		} else {
 		    /* We must have clicked a shortcut.  Put back the
@@ -137,25 +148,28 @@ char *do_browser(char *path, DIR *dir)
 
 		break;
 #endif
+
 	    case NANO_PREVLINE_KEY:
 		if (selected >= width)
 		    selected -= width;
 		break;
+
 	    case NANO_BACK_KEY:
 		if (selected > 0)
 		    selected--;
 		break;
+
 	    case NANO_NEXTLINE_KEY:
-		if (selected + width <= numents - 1)
+		if (selected + width <= filelist_len - 1)
 		    selected += width;
 		break;
+
 	    case NANO_FORWARD_KEY:
-		if (selected < numents - 1)
+		if (selected < filelist_len - 1)
 		    selected++;
 		break;
+
 	    case NANO_PREVPAGE_KEY:
-	    case NANO_PREVPAGE_FKEY:
-	    case '-': /* Pico compatibility. */
 		if (selected >= (editwinrows + fileline % editwinrows) *
 			width)
 		    selected -= (editwinrows + fileline % editwinrows) *
@@ -163,17 +177,15 @@ char *do_browser(char *path, DIR *dir)
 		else
 		    selected = 0;
 		break;
+
 	    case NANO_NEXTPAGE_KEY:
-	    case NANO_NEXTPAGE_FKEY:
-	    case ' ': /* Pico compatibility. */
 		selected += (editwinrows - fileline % editwinrows) *
 			width;
-		if (selected >= numents)
-		    selected = numents - 1;
+		if (selected >= filelist_len)
+		    selected = filelist_len - 1;
 		break;
+
 	    case NANO_HELP_KEY:
-	    case NANO_HELP_FKEY:
-	    case '?': /* Pico compatibility. */
 #ifndef DISABLE_HELP
 		do_help();
 		curs_set(0);
@@ -181,9 +193,8 @@ char *do_browser(char *path, DIR *dir)
 		nano_disabled_msg();
 #endif
 		break;
+
 	    case NANO_ENTER_KEY:
-	    case 'S': /* Pico compatibility. */
-	    case 's':
 		/* You can't move up from "/". */
 		if (strcmp(filelist[selected], "/..") == 0) {
 		    statusbar(_("Can't move up a directory"));
@@ -232,7 +243,7 @@ char *do_browser(char *path, DIR *dir)
 		path = mallocstrcpy(path, filelist[selected]);
 
 		/* Start over again with the new path value. */
-		free_chararray(filelist, numents);
+		free_chararray(filelist, filelist_len);
 		goto change_browser_directory;
 
 	    /* Redraw the screen. */
@@ -242,25 +253,44 @@ char *do_browser(char *path, DIR *dir)
 
 	    /* Go to a specific directory. */
 	    case NANO_GOTOLINE_KEY:
-	    case NANO_GOTOLINE_FKEY:
-	    case 'G': /* Pico compatibility. */
-	    case 'g':
 		curs_set(1);
 
-		j = do_prompt(FALSE, gotodir_list, "",
+		i = do_prompt(TRUE,
+#ifndef DISABLE_TABCOMP
+			FALSE,
+#endif
+			gotodir_list, ans,
 #ifndef NANO_TINY
 			NULL,
 #endif
-			_("Go To Directory"));
+			browser_refresh, _("Go To Directory"));
 
 		curs_set(0);
+#if !defined(DISABLE_HELP) || !defined(DISABLE_MOUSE)
+		currshortcut = browser_list;
+#endif
 		bottombars(browser_list);
 
-		if (j < 0) {
+		if (i < 0) {
+		    /* We canceled.  Indicate that on the statusbar, and
+		     * blank out ans, since we're done with it. */
 		    statusbar(_("Cancelled"));
+		    ans = mallocstrcpy(ans, "");
+		    break;
+		} else if (i != 0) {
+		    /* Put back the "Go to Directory" key and save
+		     * answer in ans, so that the file list is displayed
+		     * again, the prompt is displayed again, and what we
+		     * typed before at the prompt is displayed again. */
+		    unget_kbinput(NANO_GOTOLINE_KEY, FALSE, FALSE);
+		    ans = mallocstrcpy(ans, answer);
 		    break;
 		}
 
+		/* We have a directory.  Blank out ans, since we're done
+		 * with it. */
+		ans = mallocstrcpy(ans, "");
+
 		new_path = real_dir_from_tilde(answer);
 
 		if (new_path[0] != '/') {
@@ -293,14 +323,11 @@ char *do_browser(char *path, DIR *dir)
 		/* Start over again with the new path value. */
 		free(path);
 		path = new_path;
-		free_chararray(filelist, numents);
+		free_chararray(filelist, filelist_len);
 		goto change_browser_directory;
 
 	    /* Abort the browser. */
 	    case NANO_EXIT_KEY:
-	    case NANO_EXIT_FKEY:
-	    case 'E': /* Pico compatibility. */
-	    case 'e':
 		abort = TRUE;
 		break;
 	}
@@ -308,89 +335,11 @@ char *do_browser(char *path, DIR *dir)
 	if (abort)
 	    break;
 
-	blank_edit();
+	browser_refresh();
 
-	j = (width != 0) ? width * editwinrows * ((selected / width) /
-		editwinrows) : 0;
-
-	wmove(edit, 0, 0);
-
-	{
-	    size_t foo_len = mb_cur_max() * 7;
-	    char *foo = charalloc(foo_len + 1);
-
-	    for (; j < numents && editline <= editwinrows - 1; j++) {
-		char *disp = display_string(tail(filelist[j]), 0,
-			longest, FALSE);
-
-		/* Highlight the currently selected file/dir. */
-		if (j == selected)
-		    wattron(edit, A_REVERSE);
-
-		blank_line(edit, editline, col, longest);
-		mvwaddstr(edit, editline, col, disp);
-		free(disp);
-
-		col += longest;
-		filecols++;
-
-		/* Show file info also.  We don't want to report file
-		 * sizes for links, so we use lstat().  Also, stat() and
-		 * lstat() return an error if, for example, the file is
-		 * deleted while the file browser is open.  In that
-		 * case, we report "--" as the file info. */
-		if (lstat(filelist[j], &st) == -1 ||
-			S_ISLNK(st.st_mode)) {
-		    /* Aha!  It's a symlink!  Now, is it a dir?  If so,
-		     * mark it as such. */
-		    if (stat(filelist[j], &st) == 0 &&
-			S_ISDIR(st.st_mode)) {
-			strncpy(foo, _("(dir)"), foo_len);
-			foo[foo_len] = '\0';
-		    } else
-			strcpy(foo, "--");
-		} else if (S_ISDIR(st.st_mode)) {
-		    strncpy(foo, _("(dir)"), foo_len);
-		    foo[foo_len] = '\0';
-		} else if (st.st_size < (1 << 10)) /* less than 1 k. */
-		    sprintf(foo, "%4u  B", (unsigned int)st.st_size);
-		else if (st.st_size < (1 << 20)) /* less than 1 meg. */
-		    sprintf(foo, "%4u KB",
-			(unsigned int)(st.st_size >> 10));
-		else if (st.st_size < (1 << 30)) /* less than 1 gig. */
-		    sprintf(foo, "%4u MB",
-			(unsigned int)(st.st_size >> 20));
-		else
-		    sprintf(foo, "%4u GB",
-			(unsigned int)(st.st_size >> 30));
-
-		mvwaddnstr(edit, editline, col - strlen(foo), foo,
-			foo_len);
-
-		if (j == selected)
-		    wattroff(edit, A_REVERSE);
-
-		/* Add some space between the columns. */
-		col += 2;
-
-		/* If the next entry isn't going to fit on the line,
-		 * move to the next line. */
-		if (col > COLS - longest) {
-		    editline++;
-		    col = 0;
-		    if (width == 0)
-			width = filecols;
-		}
-
-		wmove(edit, editline, col);
-	    }
-
-	    free(foo);
-	}
-
-	wnoutrefresh(edit);
-    } while ((kbinput = get_kbinput(edit, &meta_key, &func_key)) !=
-	NANO_EXIT_KEY && kbinput != NANO_EXIT_FKEY);
+	kbinput = get_kbinput(edit, &meta_key, &func_key);
+	parse_browser_input(&kbinput, &meta_key, &func_key);
+    } while (kbinput != NANO_EXIT_KEY);
 
     blank_edit();
     titlebar(NULL);
@@ -400,74 +349,13 @@ char *do_browser(char *path, DIR *dir)
 	SET(CONST_UPDATE);
 
     /* Clean up. */
-    free_chararray(filelist, numents);
     free(path);
+    free(ans);
+    free_chararray(filelist, filelist_len);
 
     return retval;
 }
 
-/* Return a list of files contained in the directory path.  *longest is
- * the maximum display length of a file, up to COLS - 1 (but at least
- * 7).  *numents is the number of files.  We assume path exists and is a
- * directory.  If neither is true, we return NULL. */
-char **browser_init(const char *path, int *longest, size_t *numents, DIR
-	*dir)
-{
-    const struct dirent *nextdir;
-    char **filelist;
-    size_t i = 0, path_len;
-
-    assert(dir != NULL);
-
-    *longest = 0;
-
-    while ((nextdir = readdir(dir)) != NULL) {
-	size_t dlen;
-
-	/* Don't show the "." entry. */
-	if (strcmp(nextdir->d_name, ".") == 0)
-	   continue;
-	i++;
-
-	dlen = strlenpt(nextdir->d_name);
-	if (dlen > *longest)
-	    *longest = (dlen > COLS - 1) ? COLS - 1 : dlen;
-    }
-
-    *numents = i;
-    rewinddir(dir);
-    *longest += 10;
-
-    filelist = (char **)nmalloc(*numents * sizeof(char *));
-
-    path_len = strlen(path);
-
-    i = 0;
-
-    while ((nextdir = readdir(dir)) != NULL && i < *numents) {
-	/* Don't show the "." entry. */
-	if (strcmp(nextdir->d_name, ".") == 0)
-	   continue;
-
-	filelist[i] = charalloc(path_len + strlen(nextdir->d_name) + 1);
-	sprintf(filelist[i], "%s%s", path, nextdir->d_name);
-	i++;
-    }
-
-    /* Maybe the number of files in the directory changed between the
-     * first time we scanned and the second.  i is the actual length of
-     * filelist, so record it. */
-    *numents = i;
-    closedir(dir);
-
-    if (*longest > COLS - 1)
-	*longest = COLS - 1;
-    if (*longest < 7)
-	*longest = 7;
-
-    return filelist;
-}
-
 /* The file browser front end.  We check to see if inpath has a dir in
  * it.  If it does, we start do_browser() from there.  Otherwise, we
  * start do_browser() from the current directory. */
@@ -523,6 +411,187 @@ char *do_browse_from(const char *inpath)
     return do_browser(path, dir);
 }
 
+/* Set filelist to the list of files contained in the directory path,
+ * set filelist_len to the number of files in that list, and set longest
+ * to the width in columns of the longest filename in that list, up to
+ * COLS - 1 (but at least 7).  Assume path exists and is a directory. */
+void browser_init(const char *path, DIR *dir)
+{
+    const struct dirent *nextdir;
+    size_t i = 0, path_len;
+
+    assert(dir != NULL);
+
+    longest = 0;
+
+    while ((nextdir = readdir(dir)) != NULL) {
+	size_t d_len;
+
+	/* Don't show the "." entry. */
+	if (strcmp(nextdir->d_name, ".") == 0)
+	   continue;
+	i++;
+
+	d_len = strlenpt(nextdir->d_name);
+	if (d_len > longest)
+	    longest = (d_len > COLS - 1) ? COLS - 1 : d_len;
+    }
+
+    filelist_len = i;
+    rewinddir(dir);
+    longest += 10;
+
+    filelist = (char **)nmalloc(filelist_len * sizeof(char *));
+
+    path_len = strlen(path);
+
+    i = 0;
+
+    while ((nextdir = readdir(dir)) != NULL && i < filelist_len) {
+	/* Don't show the "." entry. */
+	if (strcmp(nextdir->d_name, ".") == 0)
+	   continue;
+
+	filelist[i] = charalloc(path_len + strlen(nextdir->d_name) + 1);
+	sprintf(filelist[i], "%s%s", path, nextdir->d_name);
+	i++;
+    }
+
+    /* Maybe the number of files in the directory changed between the
+     * first time we scanned and the second.  i is the actual length of
+     * filelist, so record it. */
+    filelist_len = i;
+    closedir(dir);
+
+    if (longest > COLS - 1)
+	longest = COLS - 1;
+    if (longest < 7)
+	longest = 7;
+}
+
+/* Determine the shortcut key corresponding to the values of kbinput
+ * (the key itself), meta_key (whether the key is a meta sequence), and
+ * func_key (whether the key is a function key), if any.  In the
+ * process, convert certain non-shortcut keys used by Pico's file
+ * browser into their corresponding shortcut keys. */
+void parse_browser_input(int *kbinput, bool *meta_key, bool *func_key)
+{
+    get_shortcut(browser_list, kbinput, meta_key, func_key);
+
+    /* Pico compatibility. */
+    if (*meta_key == FALSE && *func_key == FALSE) {
+	switch (*kbinput) {
+	    case ' ':
+		*kbinput = NANO_NEXTPAGE_KEY;
+		break;
+	    case '-':
+		*kbinput = NANO_PREVPAGE_KEY;
+		break;
+	    case '?':
+		*kbinput = NANO_HELP_KEY;
+		break;
+	    case 'E':
+	    case 'e':
+		*kbinput = NANO_EXIT_KEY;
+		break;
+	    case 'G':
+	    case 'g':
+		*kbinput = NANO_GOTOLINE_KEY;
+		break;
+	    case 'S':
+	    case 's':
+		*kbinput = NANO_ENTER_KEY;
+		break;
+	}
+    }
+}
+
+/* Calculate the number of columns needed to display the list of files
+ * in the array filelist, if necessary, and then display the list of
+ * files. */
+void browser_refresh(void)
+{
+    struct stat st;
+    int i;
+    int col = 0, filecols = 0, editline = 0;
+    size_t foo_len = mb_cur_max() * 7;
+    char *foo = charalloc(foo_len + 1);
+
+    i = (width != 0) ? width * editwinrows * ((selected / width) /
+	editwinrows) : 0;
+
+    blank_edit();
+
+    wmove(edit, 0, 0);
+
+    for (; i < filelist_len && editline <= editwinrows - 1; i++) {
+	char *disp = display_string(tail(filelist[i]), 0, longest,
+		FALSE);
+
+	/* Highlight the currently selected file/dir. */
+	if (i == selected)
+	    wattron(edit, A_REVERSE);
+
+	blank_line(edit, editline, col, longest);
+	mvwaddstr(edit, editline, col, disp);
+	free(disp);
+
+	col += longest;
+	filecols++;
+
+	/* Show file info also.  We don't want to report file sizes for
+	 * links, so we use lstat().  Also, stat() and lstat() return an
+	 * error if, for example, the file is deleted while the file
+	 * browser is open.  In that case, we report "--" as the file
+	 * info. */
+	if (lstat(filelist[i], &st) == -1 || S_ISLNK(st.st_mode)) {
+	    /* Aha!  It's a symlink!  Now, is it a dir?  If so, mark it
+	     * as such. */
+	    if (stat(filelist[i], &st) == 0 && S_ISDIR(st.st_mode)) {
+		strncpy(foo, _("(dir)"), foo_len);
+		foo[foo_len] = '\0';
+	    } else
+		strcpy(foo, "--");
+	} else if (S_ISDIR(st.st_mode)) {
+	    strncpy(foo, _("(dir)"), foo_len);
+	    foo[foo_len] = '\0';
+	} else if (st.st_size < (1 << 10)) /* less than 1 k. */
+	    sprintf(foo, "%4u  B", (unsigned int)st.st_size);
+	else if (st.st_size < (1 << 20)) /* less than 1 meg. */
+	    sprintf(foo, "%4u KB", (unsigned int)(st.st_size >> 10));
+	else if (st.st_size < (1 << 30)) /* less than 1 gig. */
+	    sprintf(foo, "%4u MB", (unsigned int)(st.st_size >> 20));
+	else
+	    sprintf(foo, "%4u GB", (unsigned int)(st.st_size >> 30));
+
+	mvwaddnstr(edit, editline, col - strlen(foo), foo, foo_len);
+
+	if (i == selected)
+	    wattroff(edit, A_REVERSE);
+
+	/* Add some space between the columns. */
+	col += 2;
+
+	/* If the next entry isn't going to fit on the line,
+	 * move to the next line. */
+	if (col > COLS - longest) {
+	    editline++;
+	    col = 0;
+
+	    /* Set the number of columns to display the list in, if
+	     * necessary, so that we don't divide by 0. */
+	    if (width == 0)
+		width = filecols;
+	}
+
+	wmove(edit, editline, col);
+    }
+
+    free(foo);
+
+    wnoutrefresh(edit);
+}
+
 /* Strip one directory from the end of path. */
 void striponedir(char *path)
 {
