@@ -1,9 +1,9 @@
-/* $Id: files.c,v 1.270 2005/12/08 07:24:54 dolorous Exp $ */
+/* $Id: files.c,v 1.276 2006/02/24 19:38:20 dolorous Exp $ */
 /**************************************************************************
  *   files.c                                                              *
  *                                                                        *
  *   Copyright (C) 1999-2004 Chris Allegretta                             *
- *   Copyright (C) 2005 David Lawrence Ramsey                             *
+ *   Copyright (C) 2005-2006 David Lawrence Ramsey                        *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
  *   the Free Software Foundation; either version 2, or (at your option)  *
@@ -713,6 +713,9 @@ void do_insertfile(
 #endif
 
 	i = do_prompt(TRUE,
+#ifndef DISABLE_TABCOMP
+		TRUE,
+#endif
 #ifndef NANO_TINY
 		execute ? extcmd_list :
 #endif
@@ -720,7 +723,7 @@ void do_insertfile(
 #ifndef NANO_TINY
 		NULL,
 #endif
-		_(msg),
+		 edit_refresh, _(msg),
 #ifndef DISABLE_OPERATINGDIR
 		operating_dir != NULL && strcmp(operating_dir, ".") != 0 ?
 		operating_dir :
@@ -1752,11 +1755,19 @@ int do_writeout(bool exiting)
 	 * and we're at the "Write File" prompt, disable tab
 	 * completion. */
 	i = do_prompt(!ISSET(RESTRICTED) ||
-		openfile->filename[0] == '\0', writefile_list, ans,
+		openfile->filename[0] == '\0',
+#ifndef DISABLE_TABCOMP
+		TRUE,
+#endif
+		writefile_list, ans,
 #ifndef NANO_TINY
-		NULL, "%s%s%s", _(msg), formatstr, backupstr
+		NULL,
+#endif
+		 edit_refresh, "%s%s%s", _(msg),
+#ifndef NANO_TINY
+		formatstr, backupstr
 #else
-		"%s", _(msg)
+		"", ""
 #endif
 		);
 
@@ -1771,8 +1782,6 @@ int do_writeout(bool exiting)
 	    if (i == NANO_TOFILES_KEY) {
 		char *tmp = do_browse_from(answer);
 
-		currshortcut = writefile_list;
-
 		if (tmp == NULL)
 		    continue;
 
@@ -1954,15 +1963,14 @@ int is_dir(const char *buf)
 {
     char *dirptr = real_dir_from_tilde(buf);
     struct stat fileinfo;
-
-    int ret = (stat(dirptr, &fileinfo) != -1 &&
+    int retval = (stat(dirptr, &fileinfo) != -1 &&
 	S_ISDIR(fileinfo.st_mode));
 
     assert(buf != NULL && dirptr != buf);
 
     free(dirptr);
 
-    return ret;
+    return retval;
 }
 
 /* These functions, username_tab_completion(), cwd_tab_completion()
@@ -2007,8 +2015,8 @@ char **username_tab_completion(const char *buf, size_t *num_matches,
 		continue;
 #endif
 
-	    matches = (char **)nrealloc(matches,
-		(*num_matches + 1) * sizeof(char *));
+	    matches = (char **)nrealloc(matches, (*num_matches + 1) *
+		sizeof(char *));
 	    matches[*num_matches] =
 		charalloc(strlen(userdata->pw_name) + 2);
 	    sprintf(matches[*num_matches], "~%s", userdata->pw_name);
@@ -2022,8 +2030,8 @@ char **username_tab_completion(const char *buf, size_t *num_matches,
 
 /* We consider the first buflen characters of buf for filename tab
  * completion. */
-char **cwd_tab_completion(const char *buf, size_t *num_matches, size_t
-	buflen)
+char **cwd_tab_completion(const char *buf, bool allow_files, size_t
+	*num_matches, size_t buflen)
 {
     char *dirname = mallocstrcpy(NULL, buf), *filename;
 #ifndef DISABLE_OPERATINGDIR
@@ -2073,36 +2081,41 @@ char **cwd_tab_completion(const char *buf, size_t *num_matches, size_t
     filenamelen = strlen(filename);
 
     while ((nextdir = readdir(dir)) != NULL) {
+	bool skip_match = FALSE;
+
 #ifdef DEBUG
 	fprintf(stderr, "Comparing \'%s\'\n", nextdir->d_name);
 #endif
 	/* See if this matches. */
 	if (strncmp(nextdir->d_name, filename, filenamelen) == 0 &&
-		(*filename == '.' ||
-		(strcmp(nextdir->d_name, ".") != 0 &&
-		strcmp(nextdir->d_name, "..") != 0))) {
+		(*filename == '.' || (strcmp(nextdir->d_name, ".") !=
+		0 && strcmp(nextdir->d_name, "..") != 0))) {
 	    /* Cool, found a match.  Add it to the list.  This makes a
 	     * lot more sense to me (Chris) this way... */
 
+	    char *tmp = charalloc(strlen(dirname) +
+		strlen(nextdir->d_name) + 1);
+	    sprintf(tmp, "%s%s", dirname, nextdir->d_name);
+
 #ifndef DISABLE_OPERATINGDIR
 	    /* ...unless the match exists outside the operating
-	     * directory, in which case just go to the next match.  To
-	     * properly do operating directory checking, we have to add
-	     * the directory name to the beginning of the proposed match
-	     * before we check it. */
-	    char *tmp2 = charalloc(strlen(dirname) +
-		strlen(nextdir->d_name) + 1);
-
-	    sprintf(tmp2, "%s%s", dirname, nextdir->d_name);
-	    if (check_operating_dir(tmp2, TRUE)) {
-		free(tmp2);
-		continue;
-	    }
-	    free(tmp2);
+	     * directory, in which case just go to the next match. */
+	    if (check_operating_dir(tmp, TRUE))
+		skip_match = TRUE;
 #endif
 
-	    matches = (char **)nrealloc(matches,
-		(*num_matches + 1) * sizeof(char *));
+	    /* ...or unless the match isn't a directory and allow_files
+	     * isn't set, in which case just go to the next match. */
+	    if (!allow_files && !is_dir(tmp))
+		skip_match = TRUE;
+
+	    free(tmp);
+
+	    if (skip_match)
+		continue;
+
+	    matches = (char **)nrealloc(matches, (*num_matches + 1) *
+		sizeof(char *));
 	    matches[*num_matches] = mallocstrcpy(NULL, nextdir->d_name);
 	    ++(*num_matches);
 	}
@@ -2116,13 +2129,15 @@ char **cwd_tab_completion(const char *buf, size_t *num_matches, size_t
 }
 
 /* Do tab completion.  place refers to how much the statusbar cursor
- * position should be advanced. */
-char *input_tab(char *buf, size_t *place, bool *lastwastab, bool *list)
+ * position should be advanced.  refresh_func is the function we will
+ * call to refresh the edit window. */
+char *input_tab(char *buf, bool allow_files, size_t *place, bool
+	*lastwastab, void (*refresh_func)(void), bool *list)
 {
     size_t num_matches = 0;
     char **matches = NULL;
 
-    assert(buf != NULL && place != NULL && *place <= strlen(buf) && lastwastab != NULL && list != NULL);
+    assert(buf != NULL && place != NULL && *place <= strlen(buf) && lastwastab != NULL && refresh_func != NULL && list != NULL);
 
     *list = FALSE;
 
@@ -2138,7 +2153,8 @@ char *input_tab(char *buf, size_t *place, bool *lastwastab, bool *list)
 
     /* Match against files relative to the current working directory. */
     if (matches == NULL)
-	matches = cwd_tab_completion(buf, &num_matches, *place);
+	matches = cwd_tab_completion(buf, allow_files, &num_matches,
+		*place);
 
     if (num_matches <= 0)
 	beep();
@@ -2279,7 +2295,7 @@ char *input_tab(char *buf, size_t *place, bool *lastwastab, bool *list)
     /* Only refresh the edit window if we don't have a list of filename
      * matches on it. */
     if (*list == FALSE)
-	edit_refresh();
+	refresh_func();
 
     /* Enable el cursor. */
     curs_set(1);
