@@ -1,9 +1,9 @@
-/* $Id: global.c,v 1.155 2005/12/08 07:09:08 dolorous Exp $ */
+/* $Id: global.c,v 1.162 2006/03/29 19:58:49 dolorous Exp $ */
 /**************************************************************************
  *   global.c                                                             *
  *                                                                        *
  *   Copyright (C) 1999-2004 Chris Allegretta                             *
- *   Copyright (C) 2005 David Lawrence Ramsey                             *
+ *   Copyright (C) 2005-2006 David Lawrence Ramsey                        *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
  *   the Free Software Foundation; either version 2, or (at your option)  *
@@ -65,6 +65,12 @@ partition *filepart = NULL;
 openfilestruct *openfile = NULL;
 	/* The list of all open file buffers. */
 
+#ifndef NANO_TINY
+char *matchbrackets = NULL;
+	/* The opening and closing brackets that can be found by bracket
+	 * searches. */
+#endif
+
 #if !defined(NANO_TINY) && defined(ENABLE_NANORC)
 char *whitespace = NULL;
 	/* The characters used when displaying the first characters of
@@ -246,16 +252,16 @@ void sc_init_one(shortcut **shortcutage, int ctrlval, const char *desc,
  * Uncut shortcut in the main shortcut list with UnJustify. */
 void shortcut_init(bool unjustify)
 {
+    const char *cancel_msg = N_("Cancel");
     const char *get_help_msg = N_("Get Help");
     const char *exit_msg = N_("Exit");
     const char *prev_page_msg = N_("Prev Page");
     const char *next_page_msg = N_("Next Page");
-    const char *replace_msg = N_("Replace");
     const char *go_to_line_msg = N_("Go To Line");
-    const char *cancel_msg = N_("Cancel");
+    const char *replace_msg = N_("Replace");
+    const char *refresh_msg = N_("Refresh");
     const char *first_line_msg = N_("First Line");
     const char *last_line_msg = N_("Last Line");
-    const char *refresh_msg = N_("Refresh");
 #ifndef NANO_TINY
     const char *cut_till_end_msg = N_("CutTillEnd");
 #endif
@@ -281,6 +287,7 @@ void shortcut_init(bool unjustify)
     const char *to_files_msg = N_("To Files");
 #endif
 #ifndef DISABLE_HELP
+    const char *nano_cancel_msg = N_("Cancel the current function");
     const char *nano_help_msg = N_("Invoke the help menu");
     const char *nano_exit_msg =
 #ifdef ENABLE_MULTIBUFFER
@@ -363,7 +370,6 @@ void shortcut_init(bool unjustify)
 #ifndef NANO_TINY
     const char *nano_bracket_msg = N_("Find matching bracket");
 #endif
-    const char *nano_cancel_msg = N_("Cancel the current function");
     const char *nano_firstline_msg =
 	N_("Go to the first line of the file");
     const char *nano_lastline_msg =
@@ -417,20 +423,19 @@ void shortcut_init(bool unjustify)
 
     /* Translators: try to keep this string under 10 characters long. */
     sc_init_one(&main_list, NANO_HELP_KEY, get_help_msg,
-	IFHELP(nano_help_msg, NANO_NO_KEY), NANO_HELP_FKEY,
-	NANO_NO_KEY, VIEW,
+	IFHELP(nano_help_msg, NANO_NO_KEY), NANO_HELP_FKEY, NANO_NO_KEY,
+	VIEW,
 #ifndef DISABLE_HELP
-		do_help
+	do_help
 #else
-		nano_disabled_msg
+	nano_disabled_msg
 #endif
-		);
+	);
 
     /* Translators: try to keep this string under 10 characters long. */
     sc_init_one(&main_list, NANO_EXIT_KEY,
 #ifdef ENABLE_MULTIBUFFER
-	openfile != NULL && openfile != openfile->next ?
-	N_("Close") :
+	openfile != NULL && openfile != openfile->next ? N_("Close") :
 #endif
 	exit_msg, IFHELP(nano_exit_msg, NANO_NO_KEY), NANO_EXIT_FKEY,
 	NANO_NO_KEY, VIEW, do_exit);
@@ -442,14 +447,14 @@ void shortcut_init(bool unjustify)
 
     /* Translators: try to keep this string under 10 characters long. */
     sc_init_one(&main_list, NANO_JUSTIFY_KEY, N_("Justify"),
-	IFHELP(nano_justify_msg, NANO_NO_KEY),
-	NANO_JUSTIFY_FKEY, NANO_NO_KEY, NOVIEW,
+	IFHELP(nano_justify_msg, NANO_NO_KEY), NANO_JUSTIFY_FKEY,
+	NANO_NO_KEY, NOVIEW,
 #ifndef DISABLE_JUSTIFY
-		do_justify_void
+	do_justify_void
 #else
-		nano_disabled_msg
+	nano_disabled_msg
 #endif
-		);
+	);
 
     /* We allow inserting files in view mode if multibuffers are
      * available, so that we can view multiple files.  If we're using
@@ -462,12 +467,11 @@ void shortcut_init(bool unjustify)
 	IFHELP(nano_insert_msg, NANO_NO_KEY), NANO_INSERTFILE_FKEY,
 	NANO_NO_KEY,
 #ifdef ENABLE_MULTIBUFFER
-		VIEW
+	VIEW
 #else
-		NOVIEW
+	NOVIEW
 #endif
-		, !ISSET(RESTRICTED) ? do_insertfile_void :
-		nano_disabled_msg);
+	, !ISSET(RESTRICTED) ? do_insertfile_void : nano_disabled_msg);
 
     /* Translators: try to keep this string under 10 characters long. */
     sc_init_one(&main_list, NANO_WHEREIS_KEY, N_("Where Is"),
@@ -486,8 +490,8 @@ void shortcut_init(bool unjustify)
 
     /* Translators: try to keep this string under 10 characters long. */
     sc_init_one(&main_list, NANO_CUT_KEY, N_("Cut Text"),
-	IFHELP(nano_cut_msg, NANO_NO_KEY), NANO_CUT_FKEY,
-	NANO_NO_KEY, NOVIEW, do_cut_text);
+	IFHELP(nano_cut_msg, NANO_NO_KEY), NANO_CUT_FKEY, NANO_NO_KEY,
+	NOVIEW, do_cut_text);
 
     if (unjustify)
     /* Translators: try to keep this string under 10 characters long. */
@@ -510,12 +514,12 @@ void shortcut_init(bool unjustify)
      * on the command line. */
     /* Translators: try to keep this string under 10 characters long. */
     sc_init_one(&main_list, NANO_SPELL_KEY, N_("To Spell"),
-		IFHELP(nano_spell_msg, NANO_NO_KEY), NANO_SPELL_FKEY,
-		NANO_NO_KEY, NOVIEW,
+	IFHELP(nano_spell_msg, NANO_NO_KEY), NANO_SPELL_FKEY,
+	NANO_NO_KEY, NOVIEW,
 #ifndef DISABLE_SPELLER
-		!ISSET(RESTRICTED) ? do_spell :
+	!ISSET(RESTRICTED) ? do_spell :
 #endif
-		nano_disabled_msg);
+	nano_disabled_msg);
 
     sc_init_one(&main_list, NANO_GOTOLINE_KEY, go_to_line_msg,
 	IFHELP(nano_gotoline_msg, NANO_GOTOLINE_ALTKEY),
@@ -545,40 +549,40 @@ void shortcut_init(bool unjustify)
 	NANO_NO_KEY, VIEW, do_down);
 
     sc_init_one(&main_list, NANO_FORWARD_KEY, N_("Forward"),
-	IFHELP(nano_forward_msg, NANO_NO_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, do_right);
+	IFHELP(nano_forward_msg, NANO_NO_KEY), NANO_NO_KEY, NANO_NO_KEY,
+	VIEW, do_right);
 
     sc_init_one(&main_list, NANO_BACK_KEY, N_("Back"),
-	IFHELP(nano_back_msg, NANO_NO_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, do_left);
+	IFHELP(nano_back_msg, NANO_NO_KEY), NANO_NO_KEY, NANO_NO_KEY,
+	VIEW, do_left);
 
     sc_init_one(&main_list, NANO_HOME_KEY, N_("Home"),
-	IFHELP(nano_home_msg, NANO_NO_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, do_home);
+	IFHELP(nano_home_msg, NANO_NO_KEY), NANO_NO_KEY, NANO_NO_KEY,
+	VIEW, do_home);
 
     sc_init_one(&main_list, NANO_END_KEY, N_("End"),
-	IFHELP(nano_end_msg, NANO_NO_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, do_end);
+	IFHELP(nano_end_msg, NANO_NO_KEY), NANO_NO_KEY, NANO_NO_KEY,
+	VIEW, do_end);
 
     sc_init_one(&main_list, NANO_REFRESH_KEY, refresh_msg,
-	IFHELP(nano_refresh_msg, NANO_NO_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, total_refresh);
+	IFHELP(nano_refresh_msg, NANO_NO_KEY), NANO_NO_KEY, NANO_NO_KEY,
+	VIEW, total_refresh);
 
     sc_init_one(&main_list, NANO_DELETE_KEY, N_("Delete"),
-	IFHELP(nano_delete_msg, NANO_NO_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, NOVIEW, do_delete);
+	IFHELP(nano_delete_msg, NANO_NO_KEY), NANO_NO_KEY, NANO_NO_KEY,
+	NOVIEW, do_delete);
 
     sc_init_one(&main_list, NANO_BACKSPACE_KEY, N_("Backspace"),
 	IFHELP(nano_backspace_msg, NANO_NO_KEY), NANO_NO_KEY,
 	NANO_NO_KEY, NOVIEW, do_backspace);
 
     sc_init_one(&main_list, NANO_TAB_KEY, N_("Tab"),
-	IFHELP(nano_tab_msg, NANO_NO_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, NOVIEW, do_tab);
+	IFHELP(nano_tab_msg, NANO_NO_KEY), NANO_NO_KEY, NANO_NO_KEY,
+	NOVIEW, do_tab);
 
     sc_init_one(&main_list, NANO_ENTER_KEY, N_("Enter"),
-	IFHELP(nano_enter_msg, NANO_NO_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, NOVIEW, do_enter);
+	IFHELP(nano_enter_msg, NANO_NO_KEY), NANO_NO_KEY, NANO_NO_KEY,
+	NOVIEW, do_enter);
 
 #ifndef NANO_TINY
     sc_init_one(&main_list, NANO_NEXTWORD_KEY, N_("Next Word"),
@@ -651,19 +655,19 @@ void shortcut_init(bool unjustify)
     free_shortcutage(&whereis_list);
 
     sc_init_one(&whereis_list, NANO_HELP_KEY, get_help_msg,
-	IFHELP(nano_help_msg, NANO_NO_KEY), NANO_HELP_FKEY,
-	NANO_NO_KEY, VIEW,
+	IFHELP(nano_help_msg, NANO_NO_KEY), NANO_HELP_FKEY, NANO_NO_KEY,
+	VIEW,
 #ifndef DISABLE_HELP
-		do_help
+	do_help
 #else
-		nano_disabled_msg
+	nano_disabled_msg
 #endif
-		);
+	);
 
     /* Translators: try to keep this string under 10 characters long. */
     sc_init_one(&whereis_list, NANO_CANCEL_KEY, cancel_msg,
-	IFHELP(nano_cancel_msg, NANO_NO_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, NULL);
+	IFHELP(nano_cancel_msg, NANO_NO_KEY), NANO_NO_KEY, NANO_NO_KEY,
+	VIEW, NULL);
 
     /* Translators: try to keep this string under 10 characters long. */
     sc_init_one(&whereis_list, NANO_FIRSTLINE_KEY, first_line_msg,
@@ -719,8 +723,8 @@ void shortcut_init(bool unjustify)
 #ifndef NANO_TINY
     /* Translators: try to keep this string under 10 characters long. */
     sc_init_one(&whereis_list, NANO_PREVLINE_KEY, history_msg,
-	IFHELP(nano_history_msg, NANO_NO_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, NULL);
+	IFHELP(nano_history_msg, NANO_NO_KEY), NANO_NO_KEY, NANO_NO_KEY,
+	VIEW, NULL);
 
     /* Translators: try to keep this string under 10 characters long. */
     sc_init_one(&whereis_list, NANO_CUTTILLEND_KEY, cut_till_end_msg,
@@ -738,18 +742,18 @@ void shortcut_init(bool unjustify)
     free_shortcutage(&replace_list);
 
     sc_init_one(&replace_list, NANO_HELP_KEY, get_help_msg,
-	IFHELP(nano_help_msg, NANO_NO_KEY), NANO_HELP_FKEY,
-	NANO_NO_KEY, VIEW,
+	IFHELP(nano_help_msg, NANO_NO_KEY), NANO_HELP_FKEY, NANO_NO_KEY,
+	VIEW,
 #ifndef DISABLE_HELP
-		do_help
+	do_help
 #else
-		nano_disabled_msg
+	nano_disabled_msg
 #endif
-		);
+	);
 
     sc_init_one(&replace_list, NANO_CANCEL_KEY, cancel_msg,
-	IFHELP(nano_cancel_msg, NANO_NO_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, NULL);
+	IFHELP(nano_cancel_msg, NANO_NO_KEY), NANO_NO_KEY, NANO_NO_KEY,
+	VIEW, NULL);
 
     sc_init_one(&replace_list, NANO_FIRSTLINE_KEY, first_line_msg,
 	IFHELP(nano_firstline_msg, NANO_NO_KEY), NANO_FIRSTLINE_FKEY,
@@ -786,25 +790,25 @@ void shortcut_init(bool unjustify)
 
 #ifndef NANO_TINY
     sc_init_one(&replace_list, NANO_PREVLINE_KEY, history_msg,
-	IFHELP(nano_history_msg, NANO_NO_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, NULL);
+	IFHELP(nano_history_msg, NANO_NO_KEY), NANO_NO_KEY, NANO_NO_KEY,
+	VIEW, NULL);
 #endif
 
     free_shortcutage(&replace_list_2);
 
     sc_init_one(&replace_list_2, NANO_HELP_KEY, get_help_msg,
-	IFHELP(nano_help_msg, NANO_NO_KEY), NANO_HELP_FKEY,
-	NANO_NO_KEY, VIEW,
+	IFHELP(nano_help_msg, NANO_NO_KEY), NANO_HELP_FKEY, NANO_NO_KEY,
+	VIEW,
 #ifndef DISABLE_HELP
-		do_help
+	do_help
 #else
-		nano_disabled_msg
+	nano_disabled_msg
 #endif
-		);
+	);
 
     sc_init_one(&replace_list_2, NANO_CANCEL_KEY, cancel_msg,
-	IFHELP(nano_cancel_msg, NANO_NO_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, NULL);
+	IFHELP(nano_cancel_msg, NANO_NO_KEY), NANO_NO_KEY, NANO_NO_KEY,
+	VIEW, NULL);
 
     sc_init_one(&replace_list_2, NANO_FIRSTLINE_KEY, first_line_msg,
 	IFHELP(nano_firstline_msg, NANO_NO_KEY), NANO_NO_KEY,
@@ -816,25 +820,25 @@ void shortcut_init(bool unjustify)
 
 #ifndef NANO_TINY
     sc_init_one(&replace_list_2, NANO_PREVLINE_KEY, history_msg,
-	IFHELP(nano_history_msg, NANO_NO_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, NULL);
+	IFHELP(nano_history_msg, NANO_NO_KEY), NANO_NO_KEY, NANO_NO_KEY,
+	VIEW, NULL);
 #endif
 
     free_shortcutage(&gotoline_list);
 
     sc_init_one(&gotoline_list, NANO_HELP_KEY, get_help_msg,
-	IFHELP(nano_help_msg, NANO_NO_KEY), NANO_HELP_FKEY,
-	NANO_NO_KEY, VIEW,
+	IFHELP(nano_help_msg, NANO_NO_KEY), NANO_HELP_FKEY, NANO_NO_KEY,
+	VIEW,
 #ifndef DISABLE_HELP
-		do_help
+	do_help
 #else
-		nano_disabled_msg
+	nano_disabled_msg
 #endif
-		);
+	);
 
     sc_init_one(&gotoline_list, NANO_CANCEL_KEY, cancel_msg,
-	IFHELP(nano_cancel_msg, NANO_NO_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, NULL);
+	IFHELP(nano_cancel_msg, NANO_NO_KEY), NANO_NO_KEY, NANO_NO_KEY,
+	VIEW, NULL);
 
     sc_init_one(&gotoline_list, NANO_FIRSTLINE_KEY, first_line_msg,
 	IFHELP(nano_firstline_msg, NANO_NO_KEY), NANO_NO_KEY,
@@ -851,18 +855,18 @@ void shortcut_init(bool unjustify)
     free_shortcutage(&writefile_list);
 
     sc_init_one(&writefile_list, NANO_HELP_KEY, get_help_msg,
-	IFHELP(nano_help_msg, NANO_NO_KEY), NANO_HELP_FKEY,
-	NANO_NO_KEY, VIEW,
+	IFHELP(nano_help_msg, NANO_NO_KEY), NANO_HELP_FKEY, NANO_NO_KEY,
+	VIEW,
 #ifndef DISABLE_HELP
-		do_help
+	do_help
 #else
-		nano_disabled_msg
+	nano_disabled_msg
 #endif
-		);
+	);
 
     sc_init_one(&writefile_list, NANO_CANCEL_KEY, cancel_msg,
-	IFHELP(nano_cancel_msg, NANO_NO_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, NULL);
+	IFHELP(nano_cancel_msg, NANO_NO_KEY), NANO_NO_KEY, NANO_NO_KEY,
+	VIEW, NULL);
 
 #ifndef DISABLE_BROWSER
     /* If we're using restricted mode, the file browser is disabled.
@@ -919,18 +923,18 @@ void shortcut_init(bool unjustify)
     free_shortcutage(&insertfile_list);
 
     sc_init_one(&insertfile_list, NANO_HELP_KEY, get_help_msg,
-	IFHELP(nano_help_msg, NANO_NO_KEY), NANO_HELP_FKEY,
-	NANO_NO_KEY, VIEW,
+	IFHELP(nano_help_msg, NANO_NO_KEY), NANO_HELP_FKEY, NANO_NO_KEY,
+	VIEW,
 #ifndef DISABLE_HELP
-		do_help
+	do_help
 #else
-		nano_disabled_msg
+	nano_disabled_msg
 #endif
-		);
+	);
 
     sc_init_one(&insertfile_list, NANO_CANCEL_KEY, cancel_msg,
-		IFHELP(nano_cancel_msg, NANO_NO_KEY), NANO_NO_KEY,
-		NANO_NO_KEY, VIEW, NULL);
+	IFHELP(nano_cancel_msg, NANO_NO_KEY), NANO_NO_KEY, NANO_NO_KEY,
+	VIEW, NULL);
 
 #ifndef DISABLE_BROWSER
     /* If we're using restricted mode, the file browser is disabled.
@@ -967,18 +971,18 @@ void shortcut_init(bool unjustify)
     free_shortcutage(&extcmd_list);
 
     sc_init_one(&extcmd_list, NANO_HELP_KEY, get_help_msg,
-	IFHELP(nano_help_msg, NANO_NO_KEY), NANO_HELP_FKEY,
-	NANO_NO_KEY, VIEW,
+	IFHELP(nano_help_msg, NANO_NO_KEY), NANO_HELP_FKEY, NANO_NO_KEY,
+	VIEW,
 #ifndef DISABLE_HELP
-		do_help
+	do_help
 #else
-		nano_disabled_msg
+	nano_disabled_msg
 #endif
-		);
+	);
 
     sc_init_one(&extcmd_list, NANO_CANCEL_KEY, cancel_msg,
-	IFHELP(nano_cancel_msg, NANO_NO_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, NULL);
+	IFHELP(nano_cancel_msg, NANO_NO_KEY), NANO_NO_KEY, NANO_NO_KEY,
+	VIEW, NULL);
 
     sc_init_one(&extcmd_list, NANO_TOOTHERINSERT_KEY, N_("Insert File"),
 	IFHELP(nano_insert_msg, NANO_NO_KEY), NANO_NO_KEY,
@@ -995,12 +999,12 @@ void shortcut_init(bool unjustify)
     free_shortcutage(&help_list);
 
     sc_init_one(&help_list, NANO_REFRESH_KEY, refresh_msg,
-	IFHELP(nano_refresh_msg, NANO_NO_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, NULL);
+	IFHELP(nano_refresh_msg, NANO_NO_KEY), NANO_NO_KEY, NANO_NO_KEY,
+	VIEW, NULL);
 
     sc_init_one(&help_list, NANO_EXIT_KEY, exit_msg,
-	IFHELP(nano_exit_msg, NANO_NO_KEY), NANO_EXIT_FKEY,
-	NANO_NO_KEY, VIEW, NULL);
+	IFHELP(nano_exit_msg, NANO_NO_KEY), NANO_EXIT_FKEY, NANO_NO_KEY,
+	VIEW, NULL);
 
     sc_init_one(&help_list, NANO_PREVPAGE_KEY, prev_page_msg,
 	IFHELP(nano_prevpage_msg, NANO_NO_KEY), NANO_PREVPAGE_FKEY,
@@ -1023,32 +1027,26 @@ void shortcut_init(bool unjustify)
     free_shortcutage(&spell_list);
 
     sc_init_one(&spell_list, NANO_HELP_KEY, get_help_msg,
-	IFHELP(nano_help_msg, NANO_NO_KEY), NANO_HELP_FKEY,
-	NANO_NO_KEY, VIEW,
+	IFHELP(nano_help_msg, NANO_NO_KEY), NANO_HELP_FKEY, NANO_NO_KEY,
+	VIEW,
 #ifndef DISABLE_HELP
-		do_help
+	do_help
 #else
-		nano_disabled_msg
+	nano_disabled_msg
 #endif
-		);
+	);
 
     sc_init_one(&spell_list, NANO_CANCEL_KEY, cancel_msg,
-	IFHELP(nano_cancel_msg, NANO_NO_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, NULL);
+	IFHELP(nano_cancel_msg, NANO_NO_KEY), NANO_NO_KEY, NANO_NO_KEY,
+	VIEW, NULL);
 #endif
 
 #ifndef DISABLE_BROWSER
     free_shortcutage(&browser_list);
 
     sc_init_one(&browser_list, NANO_HELP_KEY, get_help_msg,
-	IFHELP(nano_help_msg, NANO_NO_KEY), NANO_HELP_FKEY,
-	NANO_NO_KEY, VIEW,
-#ifndef DISABLE_HELP
-		do_help
-#else
-		nano_disabled_msg
-#endif
-		);
+	IFHELP(nano_help_msg, NANO_NO_KEY), NANO_HELP_FKEY, NANO_NO_KEY,
+	VIEW, NULL);
 
     sc_init_one(&browser_list, NANO_EXIT_KEY, exit_msg,
 	IFHELP(nano_exitbrowser_msg, NANO_NO_KEY), NANO_EXIT_FKEY,
@@ -1070,18 +1068,18 @@ void shortcut_init(bool unjustify)
     free_shortcutage(&gotodir_list);
 
     sc_init_one(&gotodir_list, NANO_HELP_KEY, get_help_msg,
-	IFHELP(nano_help_msg, NANO_NO_KEY), NANO_HELP_FKEY,
-	NANO_NO_KEY, VIEW,
+	IFHELP(nano_help_msg, NANO_NO_KEY), NANO_HELP_FKEY, NANO_NO_KEY,
+	VIEW,
 #ifndef DISABLE_HELP
-		do_help
+	do_help
 #else
-		nano_disabled_msg
+	nano_disabled_msg
 #endif
-		);
+	);
 
     sc_init_one(&gotodir_list, NANO_CANCEL_KEY, cancel_msg,
-	IFHELP(nano_cancel_msg, NANO_NO_KEY), NANO_NO_KEY,
-	NANO_NO_KEY, VIEW, NULL);
+	IFHELP(nano_cancel_msg, NANO_NO_KEY), NANO_NO_KEY, NANO_NO_KEY,
+	VIEW, NULL);
 #endif
 
     currshortcut = main_list;
