@@ -1,9 +1,9 @@
-/* $Id: nano.c,v 1.506 2005/12/08 07:09:08 dolorous Exp $ */
+/* $Id: nano.c,v 1.510 2006/03/20 04:46:48 dolorous Exp $ */
 /**************************************************************************
  *   nano.c                                                               *
  *                                                                        *
  *   Copyright (C) 1999-2004 Chris Allegretta                             *
- *   Copyright (C) 2005 David Lawrence Ramsey                             *
+ *   Copyright (C) 2005-2006 David Lawrence Ramsey                        *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
  *   the Free Software Foundation; either version 2, or (at your option)  *
@@ -34,15 +34,12 @@
 #include <errno.h>
 #include <ctype.h>
 #include <locale.h>
+#include <termios.h>
 
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
 #endif
 
-#ifdef HAVE_TERMIOS_H
-#include <termios.h>
-#endif
-
 #ifndef NANO_TINY
 #include <setjmp.h>
 #endif
@@ -1047,10 +1044,10 @@ RETSIGTYPE handle_sigwinch(int signal)
     if (result == -1)
 	return;
 
-    /* Could check whether the COLS or LINES changed, and return
-     * otherwise.  EXCEPT, that COLS and LINES are ncurses global
-     * variables, and in some cases ncurses has already updated them. 
-     * But not in all cases, argh. */
+    /* We could check whether the COLS or LINES changed, and return
+     * otherwise.  However, COLS and LINES are curses global variables,
+     * and in some cases curses has already updated them.  But not in
+     * all cases.  Argh. */
     COLS = win.ws_col;
     LINES = win.ws_row;
 
@@ -1958,16 +1955,19 @@ int main(int argc, char **argv)
 #endif
 
 #ifndef DISABLE_JUSTIFY
+    /* If punct wasn't specified, set its default value. */
     if (punct == NULL)
-	punct = mallocstrcpy(NULL, ".?!");
+	punct = mallocstrcpy(NULL, "!.?");
 
+    /* If brackets wasn't specified, set its default value. */
     if (brackets == NULL)
-	brackets = mallocstrcpy(NULL, "'\")}]>");
+	brackets = mallocstrcpy(NULL, "\"')>]}");
 
+    /* If quotestr wasn't specified, set its default value. */
     if (quotestr == NULL)
 	quotestr = mallocstrcpy(NULL,
 #ifdef HAVE_REGEX_H
-		"^([ \t]*[|>:}#])+"
+		"^([ \t]*[#:>|}])+"
 #else
 		"> "
 #endif
@@ -2003,6 +2003,12 @@ int main(int argc, char **argv)
     }
 #endif
 
+#ifndef NANO_TINY
+    /* If matchbrackets wasn't specified, set its default value. */
+    if (matchbrackets == NULL)
+	matchbrackets = mallocstrcpy(NULL, "(<[{)>]}");
+#endif
+
 #if !defined(NANO_TINY) && defined(ENABLE_NANORC)
     /* If whitespace wasn't specified, set its default value. */
     if (whitespace == NULL) {
