@@ -1,9 +1,9 @@
-/* $Id: nano.h,v 1.144 2005/12/08 07:09:08 dolorous Exp $ */
+/* $Id: nano.h,v 1.149 2006/03/30 01:18:29 dolorous Exp $ */
 /**************************************************************************
  *   nano.h                                                               *
  *                                                                        *
  *   Copyright (C) 1999-2004 Chris Allegretta                             *
- *   Copyright (C) 2005 David Lawrence Ramsey                             *
+ *   Copyright (C) 2005-2006 David Lawrence Ramsey                        *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
  *   the Free Software Foundation; either version 2, or (at your option)  *
@@ -136,8 +136,6 @@
 #define getline ngetline
 #endif
 
-#define VERMSG "GNU nano " VERSION
-
 /* If we don't have regex support, turn the color support off, as it
  * depends on the use of regexes. */
 #ifndef HAVE_REGEX_H
@@ -502,8 +500,8 @@ typedef struct rcoption {
 #define NANO_REFRESH_KEY	NANO_CONTROL_L
 #define NANO_JUSTIFY_KEY	NANO_CONTROL_J
 #define NANO_JUSTIFY_FKEY	KEY_F(4)
-#define NANO_UNJUSTIFY_KEY	NANO_UNCUT_KEY	/* Same key as UnCut. */
-#define NANO_UNJUSTIFY_FKEY	NANO_UNCUT_FKEY	/* Same key as UnCut. */
+#define NANO_UNJUSTIFY_KEY	NANO_UNCUT_KEY
+#define NANO_UNJUSTIFY_FKEY	NANO_UNCUT_FKEY
 #define NANO_PREVLINE_KEY	NANO_CONTROL_P
 #define NANO_NEXTLINE_KEY	NANO_CONTROL_N
 #define NANO_FORWARD_KEY	NANO_CONTROL_F
@@ -590,7 +588,7 @@ typedef struct rcoption {
  * counting the blank lines at their ends. */
 #define MAX_SEARCH_HISTORY 100
 
-/* The maximum number of bytes we read from a file at one time. */
+/* The maximum number of bytes buffered at one time. */
 #define MAX_BUF_SIZE 128
 
 #endif /* !NANO_H */
