@@ -1,9 +1,9 @@
-/* $Id: prompt.c,v 1.24 2005/12/08 07:09:08 dolorous Exp $ */
+/* $Id: prompt.c,v 1.34 2006/02/18 21:32:29 dolorous Exp $ */
 /**************************************************************************
  *   prompt.c                                                             *
  *                                                                        *
  *   Copyright (C) 1999-2004 Chris Allegretta                             *
- *   Copyright (C) 2005 David Lawrence Ramsey                             *
+ *   Copyright (C) 2005-2006 David Lawrence Ramsey                        *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
  *   the Free Software Foundation; either version 2, or (at your option)  *
@@ -46,9 +46,11 @@ static bool reset_statusbar_x = FALSE;
  * shortcut key, and set finished to TRUE if we're done after running
  * or trying to run a function associated with a shortcut key.  If
  * allow_funcs is FALSE, don't actually run any functions associated
- * with shortcut keys. */
+ * with shortcut keys.  refresh_func is the function we will call to
+ * refresh the edit window. */
 int do_statusbar_input(bool *meta_key, bool *func_key, bool *s_or_t,
-	bool *ran_func, bool *finished, bool allow_funcs)
+	bool *ran_func, bool *finished, bool allow_funcs, void
+	(*refresh_func)(void))
 {
     int input;
 	/* The character we read in. */
@@ -153,7 +155,7 @@ int do_statusbar_input(bool *meta_key, bool *func_key, bool *s_or_t,
 	    switch (input) {
 		/* Handle the "universal" statusbar prompt shortcuts. */
 		case NANO_REFRESH_KEY:
-		    total_refresh();
+		    refresh_func();
 		    break;
 		case NANO_HOME_KEY:
 		    do_statusbar_home();
@@ -672,7 +674,7 @@ bool find_statusbar_bracket_match(bool reverse, const char
 {
     const char *rev_start = NULL, *found = NULL;
 
-    assert(strlen(bracket_set) == 2);
+    assert(mbstrlen(bracket_set) == 2);
 
     /* rev_start might end up 1 character before the start or after the
      * end of the line.  This won't be a problem because we'll skip over
@@ -687,8 +689,8 @@ bool find_statusbar_bracket_match(bool reverse, const char
 	 * is found. */
 	found = ((rev_start > answer && *(rev_start - 1) == '\0') ||
 		rev_start < answer) ? NULL : (reverse ?
-		revstrpbrk(answer, bracket_set, rev_start) :
-		strpbrk(rev_start, bracket_set));
+		mbrevstrpbrk(answer, bracket_set, rev_start) :
+		mbstrpbrk(rev_start, bracket_set));
 
 	/* We've found a potential match. */
 	if (found != NULL)
@@ -711,48 +713,91 @@ bool find_statusbar_bracket_match(bool reverse, const char
 void do_statusbar_find_bracket(void)
 {
     size_t statusbar_x_save, pww_save;
-    const char *bracket_list = "()<>[]{}";
-	/* The list of brackets we can find matches to. */
-    const char *pos;
-	/* The location in bracket_list of the bracket at the current
+    const char *ch;
+	/* The location in matchbrackets of the bracket at the current
 	 * cursor position. */
-    char ch;
-	/* The bracket at the current cursor position. */
-    char wanted_ch;
-	/* The bracket complementing the bracket at the current cursor
-	 * position. */
-    char bracket_set[3];
+    int ch_len;
+	/* The length of ch in bytes. */
+    const char *wanted_ch;
+	/* The location in matchbrackets of the bracket complementing
+	 * the bracket at the current cursor position. */
+    int wanted_ch_len;
+	/* The length of wanted_ch in bytes. */
+    char *bracket_set;
 	/* The pair of characters in ch and wanted_ch. */
+    size_t i;
+	/* Generic loop variable. */
+    size_t matchhalf;
+	/* The number of single-byte characters in one half of
+	 * matchbrackets. */
+    size_t mbmatchhalf;
+	/* The number of multibyte characters in one half of
+	 * matchbrackets. */
     size_t count = 1;
 	/* The initial bracket count. */
     bool reverse;
 	/* The direction we search. */
+    char *found_ch;
+	/* The character we find. */
 
-    assert(strlen(bracket_list) % 2 == 0);
+    assert(mbstrlen(matchbrackets) % 2 == 0);
 
-    ch = answer[statusbar_x];
+    ch = answer + statusbar_x;
 
-    if (ch == '\0' || (pos = strchr(bracket_list, ch)) == NULL)
+    if (ch == '\0' || (ch = mbstrchr(matchbrackets, ch)) == NULL)
 	return;
 
     /* Save where we are. */
     statusbar_x_save = statusbar_x;
     pww_save = statusbar_pww;
 
-    /* If we're on an opening bracket, we want to search forwards for a
-     * closing bracket, and if we're on a closing bracket, we want to
-     * search backwards for an opening bracket. */
-    reverse = ((pos - bracket_list) % 2 != 0);
-    wanted_ch = reverse ? *(pos - 1) : *(pos + 1);
-    bracket_set[0] = ch;
-    bracket_set[1] = wanted_ch;
-    bracket_set[2] = '\0';
+    /* If we're on an opening bracket, which must be in the first half
+     * of matchbrackets, we want to search forwards for a closing
+     * bracket.  If we're on a closing bracket, which must be in the
+     * second half of matchbrackets, we want to search backwards for an
+     * opening bracket. */
+    matchhalf = 0;
+    mbmatchhalf = mbstrlen(matchbrackets) / 2;
+
+    for (i = 0; i < mbmatchhalf; i++)
+	matchhalf += parse_mbchar(matchbrackets + matchhalf, NULL,
+		NULL);
+
+    reverse = ((ch - matchbrackets) >= matchhalf);
+
+    /* If we're on an opening bracket, set wanted_ch to the character
+     * that's matchhalf characters after ch.  If we're on a closing
+     * bracket, set wanted_ch to the character that's matchhalf
+     * characters before ch. */
+    wanted_ch = ch;
+
+    while (mbmatchhalf > 0) {
+	if (reverse)
+	    wanted_ch = matchbrackets + move_mbleft(matchbrackets,
+		wanted_ch - matchbrackets);
+	else
+	    wanted_ch += move_mbright(wanted_ch, 0);
+
+	mbmatchhalf--;
+    }
+
+    ch_len = parse_mbchar(ch, NULL, NULL);
+    wanted_ch_len = parse_mbchar(wanted_ch, NULL, NULL);
+
+    /* Fill bracket_set in with the values of ch and wanted_ch. */
+    bracket_set = charalloc((mb_cur_max() * 2) + 1);
+    strncpy(bracket_set, ch, ch_len);
+    strncpy(bracket_set + ch_len, wanted_ch, wanted_ch_len);
+    null_at(&bracket_set, ch_len + wanted_ch_len);
+
+    found_ch = charalloc(mb_cur_max() + 1);
 
     while (TRUE) {
 	if (find_statusbar_bracket_match(reverse, bracket_set)) {
 	    /* If we found an identical bracket, increment count.  If we
 	     * found a complementary bracket, decrement it. */
-	    count += (answer[statusbar_x] == ch) ? 1 : -1;
+	    parse_mbchar(answer + statusbar_x, found_ch, NULL);
+	    count += (strncmp(found_ch, ch, ch_len) == 0) ? 1 : -1;
 
 	    /* If count is zero, we've found a matching bracket.  Update
 	     * the statusbar prompt and get out. */
@@ -769,6 +814,10 @@ void do_statusbar_find_bracket(void)
 	    break;
 	}
     }
+
+    /* Clean up. */
+    free(bracket_set);
+    free(found_ch);
 }
 #endif /* !NANO_TINY */
 
@@ -850,17 +899,21 @@ bool need_statusbar_horizontal_update(size_t old_pww)
 
 /* Get a string of input at the statusbar prompt.  This should only be
  * called from do_prompt(). */
-int get_prompt_string(bool allow_tabs, const char *curranswer,
+int get_prompt_string(bool allow_tabs,
+#ifndef DISABLE_TABCOMP
+	bool allow_files,
+#endif
+	const char *curranswer,
 #ifndef NANO_TINY
 	filestruct **history_list,
 #endif
-	const shortcut *s
+	void (*refresh_func)(void), const shortcut *s
 #ifndef DISABLE_TABCOMP
 	, bool *list
 #endif
 	)
 {
-    int kbinput;
+    int kbinput = ERR;
     bool meta_key, func_key, s_or_t, ran_func, finished;
     size_t curranswer_len;
 #ifndef DISABLE_TABCOMP
@@ -910,10 +963,9 @@ int get_prompt_string(bool allow_tabs, const char *curranswer,
      * to files not specified on the command line.  In this case,
      * disable all keys that would change the text if the filename isn't
      * blank and we're at the "Write File" prompt. */
-    while ((kbinput = do_statusbar_input(&meta_key, &func_key,
-	&s_or_t, &ran_func, &finished, TRUE)) != NANO_CANCEL_KEY &&
+    while ((kbinput = do_statusbar_input(&meta_key, &func_key, &s_or_t,
+	&ran_func, &finished, TRUE, refresh_func)) != NANO_CANCEL_KEY &&
 	kbinput != NANO_ENTER_KEY) {
-
 	assert(statusbar_x <= strlen(answer));
 
 #ifndef DISABLE_TABCOMP
@@ -938,8 +990,8 @@ int get_prompt_string(bool allow_tabs, const char *curranswer,
 		} else
 #endif /* !NANO_TINY */
 		if (allow_tabs)
-		    answer = input_tab(answer, &statusbar_x, &tabbed,
-			list);
+		    answer = input_tab(answer, allow_files,
+			&statusbar_x, &tabbed, refresh_func, list);
 
 		update_statusbar_line(answer, statusbar_x);
 		break;
@@ -999,6 +1051,17 @@ int get_prompt_string(bool allow_tabs, const char *curranswer,
 		}
 		break;
 #endif /* !NANO_TINY */
+#ifndef DISABLE_HELP
+	    case NANO_HELP_KEY:
+		update_statusbar_line(answer, statusbar_x);
+
+		/* This key has a shortcut list entry when it's used to
+		 * to go to the help browser, which means that finished
+		 * has been set to TRUE.  Set it back to FALSE here, so
+		 * that we aren't kicked out of the statusbar prompt. */
+		finished = FALSE;
+		break;
+#endif /* !DISABLE_HELP */
 	}
 
 	/* If we have a shortcut with an associated function, break out
@@ -1040,16 +1103,23 @@ int get_prompt_string(bool allow_tabs, const char *curranswer,
  * static prompt, which should be NULL initially, and the answer will be
  * stored in the answer global.  Returns -1 on aborted enter, -2 on a
  * blank string, and 0 otherwise, the valid shortcut key caught.
- * curranswer is any editable text that we want to put up by default.
+ * curranswer is any editable text that we want to put up by default,
+ * and refresh_func is the function we want to call to refresh the edit
+ * window.
  *
  * The allow_tabs parameter indicates whether we should allow tabs to be
- * interpreted. */
-int do_prompt(bool allow_tabs, const shortcut *s, const char
-	*curranswer,
+ * interpreted.  The allow_files parameter indicates whether we should
+ * allow all files (as opposed to just directories) to be tab
+ * completed. */
+int do_prompt(bool allow_tabs,
+#ifndef DISABLE_TABCOMP
+	bool allow_files,
+#endif
+	const shortcut *s, const char *curranswer,
 #ifndef NANO_TINY
 	filestruct **history_list,
 #endif
-	const char *msg, ...)
+	void (*refresh_func)(void), const char *msg, ...)
 {
     va_list ap;
     int retval;
@@ -1071,15 +1141,19 @@ int do_prompt(bool allow_tabs, const shortcut *s, const char
     va_end(ap);
     null_at(&prompt, actual_x(prompt, COLS - 4));
 
-    retval = get_prompt_string(allow_tabs, curranswer,
-#ifndef NANO_TINY
-		history_list,
-#endif
-		s
+    retval = get_prompt_string(allow_tabs,
 #ifndef DISABLE_TABCOMP
-		, &list
+	allow_files,
 #endif
-		);
+	curranswer,
+#ifndef NANO_TINY
+	history_list,
+#endif
+	refresh_func, s
+#ifndef DISABLE_TABCOMP
+	, &list
+#endif
+	);
 
     free(prompt);
     prompt = NULL;
@@ -1109,7 +1183,7 @@ int do_prompt(bool allow_tabs, const shortcut *s, const char
      * matches on the edit window at this point.  Make sure that they're
      * cleared off. */
     if (list)
-	edit_refresh();
+	refresh_func();
 #endif
 
     return retval;
