@@ -1,9 +1,9 @@
-/* $Id: proto.h,v 1.316 2005/12/08 07:09:08 dolorous Exp $ */
+/* $Id: proto.h,v 1.326 2006/02/18 21:32:29 dolorous Exp $ */
 /**************************************************************************
  *   proto.h                                                              *
  *                                                                        *
  *   Copyright (C) 1999-2004 Chris Allegretta                             *
- *   Copyright (C) 2005 David Lawrence Ramsey                             *
+ *   Copyright (C) 2005-2006 David Lawrence Ramsey                        *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
  *   the Free Software Foundation; either version 2, or (at your option)  *
@@ -48,6 +48,10 @@ extern filestruct *jusbuffer;
 extern partition *filepart;
 extern openfilestruct *openfile;
 
+#ifndef NANO_TINY
+extern char *matchbrackets;
+#endif
+
 #if !defined(NANO_TINY) && defined(ENABLE_NANORC)
 extern char *whitespace;
 extern int whitespace_len[2];
@@ -134,9 +138,10 @@ extern char *homedir;
 /* Public functions in browser.c. */
 #ifndef DISABLE_BROWSER
 char *do_browser(char *path, DIR *dir);
-char **browser_init(const char *path, int *longest, size_t *numents, DIR
-	*dir);
 char *do_browse_from(const char *inpath);
+void browser_init(const char *path, DIR *dir);
+void parse_browser_input(int *kbinput, bool *meta_key, bool *func_key);
+void browser_refresh(void);
 void striponedir(char *path);
 #endif
 
@@ -196,17 +201,20 @@ size_t mbstrlen(const char *s);
 size_t nstrnlen(const char *s, size_t maxlen);
 #endif
 size_t mbstrnlen(const char *s, size_t maxlen);
+#if !defined(NANO_TINY) || !defined(DISABLE_JUSTIFY)
+char *mbstrchr(const char *s, const char *c);
+#endif
 #ifndef NANO_TINY
+char *mbstrpbrk(const char *s, const char *accept);
 char *revstrpbrk(const char *s, const char *accept, const char
 	*rev_start);
+char *mbrevstrpbrk(const char *s, const char *accept, const char
+	*rev_start);
 #endif
-#ifndef DISABLE_JUSTIFY
-char *mbstrchr(const char *s, char *c);
-#ifdef ENABLE_NANORC
+#if defined(ENABLE_NANORC) && (!defined(NANO_TINY) || !defined(DISABLE_JUSTIFY))
 bool has_blank_chars(const char *s);
 bool has_blank_mbchars(const char *s);
 #endif
-#endif
 #ifdef ENABLE_UTF8
 bool is_valid_unicode(wchar_t wc);
 #endif
@@ -289,9 +297,10 @@ void free_chararray(char **array, size_t len);
 #ifndef DISABLE_TABCOMP
 char **username_tab_completion(const char *buf, size_t *num_matches,
 	size_t buflen);
-char **cwd_tab_completion(const char *buf, size_t *num_matches, size_t
-	buflen);
-char *input_tab(char *buf, size_t *place, bool *lastwastab, bool *list);
+char **cwd_tab_completion(const char *buf, bool allow_files, size_t
+	*num_matches, size_t buflen);
+char *input_tab(char *buf, bool allow_files, size_t *place, bool
+	*lastwastab, void (*refresh_func)(void), bool *list);
 #endif
 const char *tail(const char *foo);
 #if !defined(NANO_TINY) && defined(ENABLE_NANORC)
@@ -427,7 +436,8 @@ void do_output(char *output, size_t output_len, bool allow_cntrls);
 
 /* Public functions in prompt.c. */
 int do_statusbar_input(bool *meta_key, bool *func_key, bool *s_or_t,
-	bool *ran_func, bool *finished, bool allow_funcs);
+	bool *ran_func, bool *finished, bool allow_funcs, void
+	(*refresh_func)(void));
 #ifndef DISABLE_MOUSE
 bool do_statusbar_mouse(void);
 #endif
@@ -455,21 +465,28 @@ size_t get_statusbar_page_start(size_t start_col, size_t column);
 void update_statusbar_line(const char *curranswer, size_t index);
 void reset_statusbar_cursor(void);
 bool need_statusbar_horizontal_update(size_t old_pww);
-int get_prompt_string(bool allow_tabs, const char *curranswer,
+int get_prompt_string(bool allow_tabs,
+#ifndef DISABLE_TABCOMP
+	bool allow_files,
+#endif
+	const char *curranswer,
 #ifndef NANO_TINY
 	filestruct **history_list,
 #endif
-	const shortcut *s
+	void (*refresh_func)(void), const shortcut *s
 #ifndef DISABLE_TABCOMP
 	, bool *list
 #endif
 	);
-int do_prompt(bool allow_tabs, const shortcut *s, const char
-	*curranswer,
+int do_prompt(bool allow_tabs,
+#ifndef DISABLE_TABCOMP
+	bool allow_files,
+#endif
+	const shortcut *s, const char *curranswer,
 #ifndef NANO_TINY
 	filestruct **history_list,
 #endif
-	const char *msg, ...);
+	void (*refresh_func)(void), const char *msg, ...);
 void do_prompt_abort(void);
 int do_yesno_prompt(bool all, const char *msg);
 
