@@ -1,9 +1,9 @@
-/* $Id: search.c,v 1.178 2005/12/08 07:09:08 dolorous Exp $ */
+/* $Id: search.c,v 1.187 2006/03/24 04:22:45 dolorous Exp $ */
 /**************************************************************************
  *   search.c                                                             *
  *                                                                        *
  *   Copyright (C) 1999-2004 Chris Allegretta                             *
- *   Copyright (C) 2005 David Lawrence Ramsey                             *
+ *   Copyright (C) 2005-2006 David Lawrence Ramsey                        *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
  *   the Free Software Foundation; either version 2, or (at your option)  *
@@ -123,11 +123,11 @@ void search_init_globals(void)
 }
 
 /* Set up the system variables for a search or replace.  If use_answer
- * is TRUE, only set backupstring to answer.  Return -2 to run opposite
- * program (search -> replace, replace -> search), return -1 if the
- * search should be canceled (due to Cancel, Go to Line, or a failed
- * regcomp()), return 0 on success, and return 1 on rerun calling
- * program.
+ * is TRUE, only set backupstring to answer.  Return -2 to run the
+ * opposite program (search -> replace, replace -> search), return -1 if
+ * the search should be canceled (due to Cancel, a blank search string,
+ * Go to Line, or a failed regcomp()), return 0 on success, and return 1
+ * on rerun calling program.
  *
  * replacing is TRUE if we call from do_replace(), and FALSE if called
  * from do_search(). */
@@ -168,41 +168,37 @@ int search_init(bool replacing, bool use_answer)
 	buf = mallocstrcpy(NULL, "");
 
     /* This is now one simple call.  It just does a lot. */
-    i = do_prompt(FALSE, replacing ? replace_list : whereis_list,
-	backupstring,
+    i = do_prompt(FALSE,
+#ifndef DISABLE_TABCOMP
+	TRUE,
+#endif
+	replacing ? replace_list : whereis_list, backupstring,
 #ifndef NANO_TINY
 	&search_history,
 #endif
-	"%s%s%s%s%s%s", _("Search"),
-
+	edit_refresh, "%s%s%s%s%s%s", _("Search"),
 #ifndef NANO_TINY
 	/* This string is just a modifier for the search prompt; no
 	 * grammar is implied. */
 	ISSET(CASE_SENSITIVE) ? _(" [Case Sensitive]") :
 #endif
-		"",
-
+	"",
 #ifdef HAVE_REGEX_H
 	/* This string is just a modifier for the search prompt; no
 	 * grammar is implied. */
 	ISSET(USE_REGEXP) ? _(" [Regexp]") :
 #endif
-		"",
-
+	"",
 #ifndef NANO_TINY
 	/* This string is just a modifier for the search prompt; no
 	 * grammar is implied. */
 	ISSET(BACKWARDS_SEARCH) ? _(" [Backwards]") :
 #endif
-		"",
-
-	replacing ?
+	"", replacing ?
 #ifndef NANO_TINY
-		openfile->mark_set ? _(" (to replace) in selection") :
+	openfile->mark_set ? _(" (to replace) in selection") :
 #endif
-		_(" (to replace)") : "",
-
-	buf);
+	_(" (to replace)") : "", buf);
 
     /* Release buf now that we don't need it anymore. */
     free(buf);
@@ -273,25 +269,25 @@ bool findnextstr(
 	bool no_sameline, const filestruct *begin, size_t begin_x, const
 	char *needle, size_t *needle_len)
 {
-    filestruct *fileptr = openfile->current;
-    const char *rev_start = NULL, *found = NULL;
     size_t found_len;
-	/* The length of the match we found. */
+	/* The length of the match we find. */
     size_t current_x_find = 0;
-	/* The location in the current line of the match we found. */
+	/* The location in the current line of the match we find. */
     ssize_t current_y_find = openfile->current_y;
+    filestruct *fileptr = openfile->current;
+    const char *rev_start = fileptr->data, *found = NULL;
 
     /* rev_start might end up 1 character before the start or after the
      * end of the line.  This won't be a problem because strstrwrapper()
      * will return immediately and say that no match was found, and
      * rev_start will be properly set when the search continues on the
      * previous or next line. */
-    rev_start =
+    rev_start +=
 #ifndef NANO_TINY
 	ISSET(BACKWARDS_SEARCH) ?
-	fileptr->data + (openfile->current_x - 1) :
+	openfile->current_x - 1 :
 #endif
-	fileptr->data + (openfile->current_x + 1);
+	openfile->current_x + 1;
 
     /* Look for needle in the current line we're searching. */
     while (TRUE) {
@@ -343,6 +339,7 @@ bool findnextstr(
 	    return FALSE;
 	}
 
+	/* Move to the previous or next line in the file. */
 #ifndef NANO_TINY
 	if (ISSET(BACKWARDS_SEARCH)) {
 	    fileptr = fileptr->prev;
@@ -916,11 +913,15 @@ void do_replace(void)
 
     last_replace = mallocstrcpy(last_replace, "");
 
-    i = do_prompt(FALSE, replace_list_2, last_replace,
+    i = do_prompt(FALSE,
+#ifndef DISABLE_TABCOMP
+	TRUE,
+#endif
+	replace_list_2, last_replace,
 #ifndef NANO_TINY
 	&replace_history,
 #endif
-	_("Replace with"));
+	edit_refresh, _("Replace with"));
 
 #ifndef NANO_TINY
     /* Add this replace string to the replace history list.  i == 0
@@ -980,11 +981,15 @@ void do_gotolinecolumn(ssize_t line, ssize_t column, bool use_answer,
 	char *ans = mallocstrcpy(NULL, answer);
 
 	/* Ask for it. */
-	int i = do_prompt(FALSE, gotoline_list, use_answer ? ans : "",
+	int i = do_prompt(FALSE,
+#ifndef DISABLE_TABCOMP
+		TRUE,
+#endif
+		gotoline_list, use_answer ? ans : "",
 #ifndef NANO_TINY
 		NULL,
 #endif
-		_("Enter line number, column number"));
+		edit_refresh, _("Enter line number, column number"));
 
 	free(ans);
 
@@ -1076,7 +1081,7 @@ bool find_bracket_match(bool reverse, const char *bracket_set)
     const char *rev_start = NULL, *found = NULL;
     ssize_t current_y_find = openfile->current_y;
 
-    assert(strlen(bracket_set) == 2);
+    assert(mbstrlen(bracket_set) == 2);
 
     /* rev_start might end up 1 character before the start or after the
      * end of the line.  This won't be a problem because we'll skip over
@@ -1091,8 +1096,8 @@ bool find_bracket_match(bool reverse, const char *bracket_set)
     while (TRUE) {
 	found = ((rev_start > fileptr->data && *(rev_start - 1) ==
 		'\0') || rev_start < fileptr->data) ? NULL : (reverse ?
-		revstrpbrk(fileptr->data, bracket_set, rev_start) :
-		strpbrk(rev_start, bracket_set));
+		mbrevstrpbrk(fileptr->data, bracket_set, rev_start) :
+		mbstrpbrk(rev_start, bracket_set));
 
 	/* We've found a potential match. */
 	if (found != NULL)
@@ -1130,28 +1135,38 @@ void do_find_bracket(void)
 {
     filestruct *current_save;
     size_t current_x_save, pww_save;
-    const char *bracket_list = "()<>[]{}";
-	/* The list of brackets we can find matches to. */
-    const char *pos;
-	/* The location in bracket_list of the bracket at the current
+    const char *ch;
+	/* The location in matchbrackets of the bracket at the current
 	 * cursor position. */
-    char ch;
-	/* The bracket at the current cursor position. */
-    char wanted_ch;
-	/* The bracket complementing the bracket at the current cursor
-	 * position. */
-    char bracket_set[3];
+    int ch_len;
+	/* The length of ch in bytes. */
+    const char *wanted_ch;
+	/* The location in matchbrackets of the bracket complementing
+	 * the bracket at the current cursor position. */
+    int wanted_ch_len;
+	/* The length of wanted_ch in bytes. */
+    char *bracket_set;
 	/* The pair of characters in ch and wanted_ch. */
+    size_t i;
+	/* Generic loop variable. */
+    size_t matchhalf;
+	/* The number of single-byte characters in one half of
+	 * matchbrackets. */
+    size_t mbmatchhalf;
+	/* The number of multibyte characters in one half of
+	 * matchbrackets. */
     size_t count = 1;
 	/* The initial bracket count. */
     bool reverse;
 	/* The direction we search. */
+    char *found_ch;
+	/* The character we find. */
 
-    assert(strlen(bracket_list) % 2 == 0);
+    assert(mbstrlen(matchbrackets) % 2 == 0);
 
-    ch = openfile->current->data[openfile->current_x];
+    ch = openfile->current->data + openfile->current_x;
 
-    if (ch == '\0' || (pos = strchr(bracket_list, ch)) == NULL) {
+    if (ch == '\0' || (ch = mbstrchr(matchbrackets, ch)) == NULL) {
 	statusbar(_("Not a bracket"));
 	return;
     }
@@ -1161,21 +1176,54 @@ void do_find_bracket(void)
     current_x_save = openfile->current_x;
     pww_save = openfile->placewewant;
 
-    /* If we're on an opening bracket, we want to search forwards for a
-     * closing bracket, and if we're on a closing bracket, we want to
-     * search backwards for an opening bracket. */
-    reverse = ((pos - bracket_list) % 2 != 0);
-    wanted_ch = reverse ? *(pos - 1) : *(pos + 1);
-    bracket_set[0] = ch;
-    bracket_set[1] = wanted_ch;
-    bracket_set[2] = '\0';
+    /* If we're on an opening bracket, which must be in the first half
+     * of matchbrackets, we want to search forwards for a closing
+     * bracket.  If we're on a closing bracket, which must be in the
+     * second half of matchbrackets, we want to search backwards for an
+     * opening bracket. */
+    matchhalf = 0;
+    mbmatchhalf = mbstrlen(matchbrackets) / 2;
+
+    for (i = 0; i < mbmatchhalf; i++)
+	matchhalf += parse_mbchar(matchbrackets + matchhalf, NULL,
+		NULL);
+
+    reverse = ((ch - matchbrackets) >= matchhalf);
+
+    /* If we're on an opening bracket, set wanted_ch to the character
+     * that's matchhalf characters after ch.  If we're on a closing
+     * bracket, set wanted_ch to the character that's matchhalf
+     * characters before ch. */
+    wanted_ch = ch;
+
+    while (mbmatchhalf > 0) {
+	if (reverse)
+	    wanted_ch = matchbrackets + move_mbleft(matchbrackets,
+		wanted_ch - matchbrackets);
+	else
+	    wanted_ch += move_mbright(wanted_ch, 0);
+
+	mbmatchhalf--;
+    }
+
+    ch_len = parse_mbchar(ch, NULL, NULL);
+    wanted_ch_len = parse_mbchar(wanted_ch, NULL, NULL);
+
+    /* Fill bracket_set in with the values of ch and wanted_ch. */
+    bracket_set = charalloc((mb_cur_max() * 2) + 1);
+    strncpy(bracket_set, ch, ch_len);
+    strncpy(bracket_set + ch_len, wanted_ch, wanted_ch_len);
+    null_at(&bracket_set, ch_len + wanted_ch_len);
+
+    found_ch = charalloc(mb_cur_max() + 1);
 
     while (TRUE) {
 	if (find_bracket_match(reverse, bracket_set)) {
 	    /* If we found an identical bracket, increment count.  If we
 	     * found a complementary bracket, decrement it. */
-	    count += (openfile->current->data[openfile->current_x] ==
-		ch) ? 1 : -1;
+	    parse_mbchar(openfile->current->data + openfile->current_x,
+		found_ch, NULL);
+	    count += (strncmp(found_ch, ch, ch_len) == 0) ? 1 : -1;
 
 	    /* If count is zero, we've found a matching bracket.  Update
 	     * the screen and get out. */
@@ -1193,6 +1241,10 @@ void do_find_bracket(void)
 	    break;
 	}
     }
+
+    /* Clean up. */
+    free(bracket_set);
+    free(found_ch);
 }
 
 #ifdef ENABLE_NANORC
