@@ -1,9 +1,9 @@
-/* $Id: text.c,v 1.106 2005/12/08 07:24:54 dolorous Exp $ */
+/* $Id: text.c,v 1.112 2006/02/18 21:32:29 dolorous Exp $ */
 /**************************************************************************
  *   text.c                                                               *
  *                                                                        *
  *   Copyright (C) 1999-2004 Chris Allegretta                             *
- *   Copyright (C) 2005 David Lawrence Ramsey                             *
+ *   Copyright (C) 2005-2006 David Lawrence Ramsey                        *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
  *   the Free Software Foundation; either version 2, or (at your option)  *
@@ -152,7 +152,8 @@ void do_delete(void)
 	update_line(openfile->current, openfile->current_x);
 }
 
-/* Backspace over one character. */
+/* Backspace over one character.  That is, move the cursor left one
+ * character, and then delete the character there. */
 void do_backspace(void)
 {
     if (openfile->current != openfile->fileage ||
@@ -746,8 +747,8 @@ void justify_format(filestruct *paragraph, size_t skip)
     while (*end != '\0') {
 	int end_len;
 
-	/* If this character is blank, make sure that it's a space with
-	 * no blanks after it. */
+	/* If this character is blank, change it to a space if
+	 * necessary, and skip over all blanks after it. */
 	if (is_blank_mbchar(end)) {
 	    end_len = parse_mbchar(end, NULL, NULL);
 
@@ -770,9 +771,9 @@ void justify_format(filestruct *paragraph, size_t skip)
 #endif
 	    }
 	/* If this character is punctuation optionally followed by a
-	 * bracket and then followed by blanks, make sure there are no
-	 * more than two blanks after it, and make sure that the blanks
-	 * are spaces. */
+	 * bracket and then followed by blanks, change no more than two
+	 * of the blanks to spaces if necessary, and skip over all
+	 * blanks after them. */
 	} else if (mbstrchr(punct, end) != NULL) {
 	    end_len = parse_mbchar(end, NULL, NULL);
 
@@ -825,7 +826,7 @@ void justify_format(filestruct *paragraph, size_t skip)
 #endif
 	    }
 	/* If this character is neither blank nor punctuation, leave it
-	 * alone. */
+	 * unchanged. */
 	} else {
 	    end_len = parse_mbchar(end, NULL, NULL);
 
@@ -842,7 +843,7 @@ void justify_format(filestruct *paragraph, size_t skip)
 
     *new_end = *end;
 
-    /* Make sure that there are no spaces at the end of the line. */
+    /* If there are spaces at the end of the line, remove them. */
     while (new_end > new_paragraph_data + skip &&
 	*(new_end - 1) == ' ') {
 	new_end--;
@@ -1667,11 +1668,15 @@ bool do_int_spell_fix(const char *word)
 	    do_replace_highlight(TRUE, exp_word);
 
 	    /* Allow all instances of the word to be corrected. */
-	    canceled = (do_prompt(FALSE, spell_list, word,
-#ifndef NANO_TINY
-			NULL,
+	    canceled = (do_prompt(FALSE,
+#ifndef DISABLE_TABCOMP
+		TRUE,
 #endif
-			 _("Edit a replacement")) == -1);
+		spell_list, word,
+#ifndef NANO_TINY
+		NULL,
+#endif
+		edit_refresh, _("Edit a replacement")) == -1);
 
 	    do_replace_highlight(FALSE, exp_word);
 
