@@ -1,9 +1,9 @@
-/* $Id: utils.c,v 1.110 2005/12/23 03:15:12 dolorous Exp $ */
+/* $Id: utils.c,v 1.112 2006/01/09 03:23:29 dolorous Exp $ */
 /**************************************************************************
  *   utils.c                                                              *
  *                                                                        *
  *   Copyright (C) 1999-2004 Chris Allegretta                             *
- *   Copyright (C) 2005 David Lawrence Ramsey                             *
+ *   Copyright (C) 2005-2006 David Lawrence Ramsey                        *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
  *   the Free Software Foundation; either version 2, or (at your option)  *
@@ -81,9 +81,9 @@ bool parse_num(const char *str, ssize_t *val)
     return TRUE;
 }
 
-/* Read an int and a ssize_t, separated by a comma, from str, and store
- * them in *line and *column (if they're not both NULL).  On error, we
- * return FALSE.  Otherwise, we return TRUE. */
+/* Read two ssize_t's, separated by a comma, from str, and store them in
+ * *line and *column (if they're not both NULL).  Return FALSE on error,
+ * or TRUE otherwise. */
 bool parse_line_column(const char *str, ssize_t *line, ssize_t *column)
 {
     bool retval = TRUE;
@@ -197,7 +197,10 @@ ssize_t ngetdelim(char **lineptr, size_t *n, int delim, FILE *stream)
     int c;
 
     /* Sanity checks. */
-    assert(lineptr != NULL && n != NULL && stream != NULL);
+    if (lineptr == NULL || n == NULL || stream == NULL) {
+	errno = EINVAL;
+	return -1;
+    }
 
     /* Allocate the line the first time. */
     if (*lineptr == NULL) {
