@@ -1,9 +1,9 @@
-/* $Id: winio.c,v 1.470 2005/12/08 07:24:54 dolorous Exp $ */
+/* $Id: winio.c,v 1.483 2006/03/30 01:18:29 dolorous Exp $ */
 /**************************************************************************
  *   winio.c                                                              *
  *                                                                        *
  *   Copyright (C) 1999-2004 Chris Allegretta                             *
- *   Copyright (C) 2005 David Lawrence Ramsey                             *
+ *   Copyright (C) 2005-2006 David Lawrence Ramsey                        *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
  *   the Free Software Foundation; either version 2, or (at your option)  *
@@ -27,7 +27,6 @@
 #include <stdarg.h>
 #include <string.h>
 #include <unistd.h>
-#include <errno.h>
 #include <ctype.h>
 
 static int *key_buffer = NULL;
@@ -123,6 +122,7 @@ void reset_kbinput(void)
 void get_key_buffer(WINDOW *win)
 {
     int input;
+    size_t errcount;
 
     /* If the keystroke buffer isn't empty, get out. */
     if (key_buffer != NULL)
@@ -137,14 +137,19 @@ void get_key_buffer(WINDOW *win)
      * screen updates. */
     doupdate();
 
+    errcount = 0;
     while ((input = wgetch(win)) == ERR) {
-	/* If errno is EIO, it means that the input source that we were
-	 * using is gone, so die gracefully. */
-	if (errno == EIO)
+	errcount++;
+
+	/* If we've failed to get a character MAX_BUF_SIZE times in a
+	 * row, assume that the input source we were using is gone and
+	 * die gracefully.  We could check if errno is set to EIO
+	 * ("Input/output error") and die gracefully in that case, but
+	 * it's not always set properly.  Argh. */
+	if (errcount == MAX_BUF_SIZE)
 	    handle_hupterm(0);
     }
 
-
 #ifndef NANO_TINY
     allow_pending_sigwinch(FALSE);
 #endif
@@ -469,10 +474,36 @@ int parse_kbinput(WINDOW *win, bool *meta_key, bool *func_key
 			    retval = NANO_END_KEY;
 			    break;
 #endif
-#ifdef KEY_SUSPEND
-			/* Slang doesn't support KEY_SUSPEND. */
-			case KEY_SUSPEND:
-			    retval = NANO_SUSPEND_KEY;
+#ifdef KEY_SBEG
+			/* Slang doesn't support KEY_SBEG. */
+			case KEY_SBEG:	/* Center (5) on numeric keypad
+					 * with NumLock off. */
+			    break;
+#endif
+#ifdef KEY_SDC
+			/* Slang doesn't support KEY_SDC. */
+			case KEY_SDC:
+			    retval = ISSET(REBIND_DELETE) ?
+				NANO_BACKSPACE_KEY : NANO_DELETE_KEY;
+			    break;
+#endif
+#ifdef KEY_SEND
+			/* HP-UX 10 and 11 don't support KEY_SEND. */
+			case KEY_SEND:
+			    retval = NANO_END_KEY;
+			    break;
+#endif
+#ifdef KEY_SHOME
+			/* HP-UX 10 and 11 and Slang don't support
+			 * KEY_SHOME. */
+			case KEY_SHOME:
+			    retval = NANO_HOME_KEY;
+			    break;
+#endif
+#ifdef KEY_SIC
+			/* Slang doesn't support KEY_SIC. */
+			case KEY_SIC:
+			    retval = NANO_INSERTFILE_KEY;
 			    break;
 #endif
 #ifdef KEY_SLEFT
@@ -486,6 +517,18 @@ int parse_kbinput(WINDOW *win, bool *meta_key, bool *func_key
 			case KEY_SRIGHT:
 			    retval = NANO_FORWARD_KEY;
 			    break;
+#endif
+#ifdef KEY_SSUSPEND
+			/* Slang doesn't support KEY_SSUSPEND. */
+			case KEY_SSUSPEND:
+			    retval = NANO_SUSPEND_KEY;
+			    break;
+#endif
+#ifdef KEY_SUSPEND
+			/* Slang doesn't support KEY_SUSPEND. */
+			case KEY_SUSPEND:
+			    retval = NANO_SUSPEND_KEY;
+			    break;
 #endif
 			default:
 			    retval = *kbinput;
@@ -1753,8 +1796,7 @@ char *display_string(const char *buf, size_t start_col, size_t len, bool
 	/* The string we return. */
     size_t index;
 	/* Current position in converted. */
-
-    char *buf_mb = charalloc(mb_cur_max());
+    char *buf_mb;
     int buf_mb_len;
 
     /* If dollars is TRUE, make room for the "$" at the end of the
@@ -1765,15 +1807,29 @@ char *display_string(const char *buf, size_t start_col, size_t len, bool
     if (len == 0)
 	return mallocstrcpy(NULL, "");
 
+    buf_mb = charalloc(mb_cur_max());
+
     start_index = actual_x(buf, start_col);
     column = strnlenpt(buf, start_index);
 
     assert(column <= start_col);
 
-    /* Allocate enough space for the entire line. */
-    alloc_len = (mb_cur_max() * (COLS + 1));
+    /* Make sure there's enough room for the initial character, whether
+     * it's a multibyte control character, a non-control multibyte
+     * character, a tab character, or a null terminator.  Rationale:
+     *
+     * multibyte control character followed by a null terminator:
+     *     1 byte ('^') + mb_cur_max() bytes + 1 byte ('\0')
+     * multibyte non-control character followed by a null terminator:
+     *     mb_cur_max() bytes + 1 byte ('\0')
+     * tab character followed by a null terminator:
+     *     mb_cur_max() bytes + (tabsize - 1) bytes + 1 byte ('\0')
+     *
+     * Since tabsize has a minimum value of 1, it can substitute for 1
+     * byte above. */
+    alloc_len = (mb_cur_max() + tabsize + 1) * MAX_BUF_SIZE;
+    converted = charalloc(alloc_len);
 
-    converted = charalloc(alloc_len + 1);
     index = 0;
 
     if (buf[start_index] != '\t' && (column < start_col || (dollars &&
@@ -1810,9 +1866,17 @@ char *display_string(const char *buf, size_t start_col, size_t len, bool
 #endif
     }
 
-    while (index < alloc_len - 1 && buf[start_index] != '\0') {
+    while (buf[start_index] != '\0') {
 	buf_mb_len = parse_mbchar(buf + start_index, buf_mb, NULL);
 
+	/* Make sure there's enough room for the next character, whether
+	 * it's a multibyte control character, a non-control multibyte
+	 * character, a tab character, or a null terminator. */
+	if (index + mb_cur_max() + tabsize + 1 >= alloc_len - 1) {
+	    alloc_len += (mb_cur_max() + tabsize + 1) * MAX_BUF_SIZE;
+	    converted = charealloc(converted, alloc_len);
+	}
+
 	/* If buf contains a tab character, interpret it. */
 	if (*buf_mb == '\t') {
 #if !defined(NANO_TINY) && defined(ENABLE_NANORC)
@@ -1884,8 +1948,10 @@ char *display_string(const char *buf, size_t start_col, size_t len, bool
 
     free(buf_mb);
 
-    if (index < alloc_len - 1)
-	converted[index] = '\0';
+    assert(alloc_len >= index + 1);
+
+    /* Null terminate converted. */
+    converted[index] = '\0';
 
     /* Make sure converted takes up no more than len columns. */
     index = actual_x(converted, len);
@@ -1904,7 +1970,7 @@ void titlebar(const char *path)
 {
     int space = COLS;
 	/* The space we have available for display. */
-    size_t verlen = strlenpt(VERMSG) + 1;
+    size_t verlen = strlenpt(PACKAGE_STRING) + 1;
 	/* The length of the version message in columns, plus one for
 	 * padding. */
     const char *prefix;
@@ -1944,7 +2010,8 @@ void titlebar(const char *path)
     if (space >= 4) {
 	/* Add a space after the version message, and account for both
 	 * it and the two spaces before it. */
-	mvwaddnstr(topwin, 0, 2, VERMSG, actual_x(VERMSG, verlen));
+	mvwaddnstr(topwin, 0, 2, PACKAGE_STRING,
+		actual_x(PACKAGE_STRING, verlen));
 	verlen += 3;
 
 	/* Account for the full length of the version message. */
@@ -2772,6 +2839,11 @@ void edit_redraw(const filestruct *old_current, size_t old_pww)
 
 	openfile->edittop = old_edittop;
 
+	/* Update old_current if we're not on the first page and/or
+	 * we're not on the same page as before. */
+	if (do_redraw)
+	    update_line(old_current, 0);
+
 	/* Scroll the edit window up or down until edittop is in range
 	 * of current. */
 	if (nlines < 0)
